﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace TSP
{
    public class Game1 : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        private Texture2D circleTexture;
        private Texture2D pixelTexture;
        private SpriteFont font;
        List<Vector2> routeRandom = new List<Vector2>();
        List<Vector2> routeNearest = new List<Vector2>();
        private double routeRandomLength = 0;
        private double routeNearestLength = 0;
        Random rnd = new Random();
        private KeyboardState prevState;
        private int state = 0;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferHeight = 720;
            graphics.PreferredBackBufferWidth = 1280;
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            // Random 
            for (int i = 0; i < 50; i++)
                routeRandom.Add(new Vector2(rnd.Next(20, 1260), rnd.Next(40, 700)));

            for (int i = 0; i < routeRandom.Count; i++)
                routeRandomLength += (routeRandom[i] - routeRandom[(i + 1) % routeRandom.Count]).Length();

            //Nearest neighbour
            List<Vector2> tmp = new List<Vector2>(routeRandom);
            routeNearest.Add(tmp[0]);
            tmp.RemoveAt(0);

            while (tmp.Count > 0)
            {
                //Find closest
                var start = routeNearest[routeNearest.Count - 1];
                var pick = tmp[0];
                var length = (start - pick).LengthSquared();

                for (int i = 1; i < tmp.Count; i++)
                {
                    if ((start - tmp[i]).LengthSquared() < length)
                    {
                        length = (start - tmp[i]).LengthSquared();
                        pick = tmp[i];
                    }
                }
                routeNearest.Add(pick);
                tmp.Remove(pick);
            }

            for (int i = 0; i < routeNearest.Count; i++)
                routeNearestLength += (routeNearest[i] - routeNearest[(i + 1) % routeNearest.Count]).Length();

            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            circleTexture = Content.Load<Texture2D>("circle");
            pixelTexture = Content.Load<Texture2D>("pixel");
            font = Content.Load<SpriteFont>("font");
        }

        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed 
                || Keyboard.GetState().IsKeyDown(Keys.Escape))
                Exit();

            KeyboardState kbState = Keyboard.GetState();
            if (kbState.IsKeyDown(Keys.Space) && !prevState.IsKeyDown(Keys.Space))
            {
                state = (state == 1) ? 0 : state + 1;
            }
            prevState = kbState;

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();

            if (state == 0)
            {
                for (int i = 0; i < routeRandom.Count; i++)
                    DrawLine(routeRandom[i], routeRandom[(i + 1)%routeRandom.Count], Color.GreenYellow, 3);

                spriteBatch.DrawString(font, string.Format("Random: {0:0.0}", routeRandomLength), new Vector2(10, 10),
                    Color.White);
            }
            else
            {
                for (int i = 0; i < routeNearest.Count; i++)
                    DrawLine(routeNearest[i], routeNearest[(i + 1) % routeNearest.Count], Color.GreenYellow, 3);

                spriteBatch.DrawString(font, string.Format("Nearest: {0:0.0}", routeNearestLength), new Vector2(10, 10),
                    Color.White);
            }

            foreach (var place in routeRandom)
                DrawCircle(place, Color.Blue, 25);
            DrawCircle(routeRandom[0], Color.Red, 25);

            spriteBatch.End();

            base.Draw(gameTime);
        }


        public void DrawLine(Vector2 pos1, Vector2 pos2, Color color, int width = 1)
        {
            Vector2 diff = (pos1 - pos2);
            float length = diff.Length();
            float angle = (float)Math.Atan2(diff.Y, diff.X) + MathHelper.Pi;

            spriteBatch.Draw(pixelTexture, new Rectangle((int)(pos1.X), (int)(pos1.Y), (int)length, width),
                null, color, angle, new Vector2(0, width / 2), SpriteEffects.None, 0f);
        }

        public void DrawCircle(Vector2 pos, Color color, int width = 10)
        {
            float scale = width / (float)circleTexture.Width;
            spriteBatch.Draw(circleTexture, pos, null, color, 0f,
                new Vector2(circleTexture.Width * 0.5f, circleTexture.Height * 0.5f), scale, SpriteEffects.None, 0f);
        }
    }
}
